/* -*- compile-command: "swig -c++ -java giac.i && g++ -c -fpic giac_wrap.cxx  -I/usr/lib/jvm/java-1.5.0-gcj-4.6/include -I/usr/lib/jvm/java-1.5.0-gcj-4.6/include/linux && g++ -shared giac_wrap.o -lgiac -o libgiacjava.so" -*- */
// Mac commands: swig -c++ -java giac.i && g++ -m32 -fPIC -c giac_wrap.cxx  -I/System/Library/Frameworks/JavaVM.framework/Headers -I/System/Library/Frameworks/JavaVM.framework/Headers &&  g++ -m32 -shared giac_wrap.o -L.libs -lgiac -dynamiclib -framework JavaVM  -o libgiacjava.jnilib
// or link with .libs/libgiac.a -lmpfr -lntl -lgmp -lcocoa -lpari -llapack -lblas -lgsl -lgslcblas -lintl -lpng -lz
/* File : giac.i 
   Interface for giac using swig
   Assumes gen are stored on 64 bits
 */
%module giac

%{
#include "giac.h"
%}

%javaconst(1);
namespace giac {
  enum gen_unary_types {
    // immediate type (without mem allocation) should be < _ZINT
    _INT_= 0, // int val
    _DOUBLE_= 1, // double _DOUBLE_val
    // all type below or equal to _DOUBLE_ must be non pointers
    _ZINT= 2, // mpz_t * _ZINTptr
    _REAL= 3, // mpf_t * _REALptr
    // all type strictly below _CPLX must be real types
    _CPLX= 4, // gen * _CPLXptr
    _POLY= 5, // polynome * _POLYptr
    _IDNT= 6, // identificateur * _IDNTptr
    _VECT= 7, // vecteur * _VECTptr
    _SYMB= 8, // symbolic * _SYMBptr
    _SPOL1= 9, // sparse_poly1 * _SPOL1ptr
    _FRAC= 10, // fraction * _FRACptr
    _EXT= 11, // gen * _EXTptr
    _STRNG= 12, // string * _STRNGptr
    _FUNC= 13, // unary_fonction_ptr * _FUNCptr
    _ROOT= 14, // real_complex_rootof *_ROOTptr
    _MOD= 15, // gen * _MODptr
    _USER= 16, // gen_user * _USERptr
    _MAP=17, // map<gen.gen> * _MAPptr
    _EQW=18, // eqwdata * _EQWptr
    _GROB=19, // grob * _GROBptr
    _POINTER_=20, // void * _POINTER_val
    _FLOAT_=21 // immediate, _FLOAT_val
  } ;

}
%include "std_vector.i"
%include "std_string.i"

namespace std {
  %template(vectori) vector<int>;
   %template(vectord) vector<double>;
   %template(vectorg) vector<giac::gen>;
};

namespace giac {

  class context {
  public:
    giac::sym_tab * tabptr ;
    context * globalcontextptr ;
    context * previous ;
    giac::global * globalptr; 
    const context * parent;
    giac::vecteur * quoted_global_vars;
    giac::vecteur * history_in_ptr, * history_out_ptr;
    context();
    context(const context & c);
#ifndef RTOS_THREADX
#ifndef BESTA_OS
    context(const std::string & name);
#endif
#endif
    ~context();
    context * clone() const;
  };

  class gen {
  public:
#ifdef GIAC_TYPE_ON_8BITS
    unsigned char type;  // see dispatch.h
#else
    unsigned char type:5;  // 32 types is enough, keep 3 bits more for double
    unsigned char type_unused:3; 
#endif
    signed char subtype;
    unsigned short reserved; // used if SMARTPTR is defined on 64 bit CPU (16 bits for pointer val)
    int val;
  
    inline volatile int & ref_count() const { 
#ifdef SMARTPTR64
      return ((ref_mpz_t *) ((* (longlong *) (this))>>16))->ref_count;
#else
      return __ZINTptr->ref_count;
#endif
    }
    gen(): type(_INT_),subtype(0),val(0) {
#ifdef COMPILE_FOR_STABILITY
      control_c();
#endif
    };
#ifdef SMARTPTR64
    gen(void *ptr,short int subt)  {
#ifdef COMPILE_FOR_STABILITY
      control_c();
#endif
      longlong __POINTERptr = (longlong ) new ref_void_pointer(ptr); 
#ifndef NO_STDEXCEPT
      if (__POINTERptr & 0xffff000000000000)
	setsizeerr(gettext("Pointer out of range"));
#endif
      * ((longlong *) this) = __POINTERptr << 16;
      subtype=subt;
      type=_POINTER_;
    };
#else
    gen(void *ptr,short int subt): type(_POINTER_),subtype(char(subt)) {
#ifdef COMPILE_FOR_STABILITY
      control_c();
#endif
      __POINTERptr=new ref_void_pointer(ptr); 
    };
#endif
    gen(int i): type(_INT_),subtype(0),val(i) {
#ifdef COMPILE_FOR_STABILITY
      control_c();
#endif
    };
    gen(size_t i): type(_INT_),subtype(0),val((int)i)  {
#ifdef COMPILE_FOR_STABILITY
      control_c();
#endif
    };
    gen(longlong i);
#ifdef INT128
    gen(int128_t i);
#endif
#ifdef DOUBLEVAL
    gen(double d): type(_DOUBLE_),_DOUBLE_val(d) {};
#else
    // Warning this does not work on ia64 with -O2
    gen(double d) { *((double *) this) = d; type=_DOUBLE_; };
#endif
    gen(const giac_float & f);
#ifdef BCD
    gen(accurate_bcd_float * b);
#endif
    // inline
    double DOUBLE_val() const ;
    giac_float FLOAT_val() const ;
    gen(int a,int b);
    gen(double a,double b);
    gen(const gen & a,const gen & b);
    gen(const std::complex<double> & c);
    gen(const gen & e);
    gen (const giac::identificateur & s);
    gen (giac::ref_identificateur * sptr);
    gen (const giac::vecteur & v,short int s=0);
    gen (giac::ref_vecteur * vptr,short int s=0); 
    // vptr must be a pointer allocated by new, do not delete it explicitly
    gen (const giac::symbolic & s);
    gen (giac::ref_symbolic * sptr);
    gen (const giac::gen_user & g);
    gen (const giac::real_object & g);
    // Pls do not use this constructor unless you know exactly what you do
    gen (giac::Tref_tensor<gen> * pptr);
    gen (const giac::polynome & p);
    gen (const giac::fraction & p);
    gen (const std::string & s,giac::context * contextptr);
    // gen (const wchar_t * s,giac::context * contextptr);
    // gen (const char * s,giac::context * contextptr);
    // gen (const giac::sparse_poly1 & p);
    gen (const giac::unary_function_ptr & f,int nargs=1);
    // gen (const giac::unary_function_ptr * f,int nargs=1);
    gen (const giac::gen_map & m);
    gen (const giac::eqwdata & );
    gen (const giac::grob & );
#ifdef HAVE_GMPXX_H
    gen (const giac::mpz_class &);
#endif
    gen (const giac::my_mpz &);
    ~gen();
    inline mpz_t * ref_ZINTptr() const ;
    inline giac::real_object * ref_REALptr() const ;
    inline giac::gen * ref_CPLXptr() const ;
    inline giac::identificateur * ref_IDNTptr() const ;
    inline giac::symbolic * ref_SYMBptr() const ;
    inline giac::gen * ref_MODptr () const ;
    inline giac::Tfraction<gen> * ref_FRACptr() const ;
    inline giac::gen * ref_EXTptr () const ;
    inline giac::polynome * ref_POLYptr() const ;
    inline giac::vecteur * ref_VECTptr() const ;
    inline giac::sparse_poly1 * ref_SPOL1ptr() const ;
    inline std::string * ref_STRNGptr() const ;
    inline giac::unary_function_ptr * ref_FUNCptr() const ;
    inline giac::gen_user * ref_USERptr() const ;
    inline giac::gen_map * ref_MAPptr() const ;
    inline giac::eqwdata * ref_EQWptr() const ;
    inline giac::grob * ref_GROBptr() const ;
    inline void * ref_POINTER_val() const ;

    bool in_eval(int level,gen & evaled,const giac::context * contextptr) const;
    gen eval(int level,const giac::context * contextptr) const;
    // inline gen eval() const { return eval(DEFAULT_EVAL_LEVEL,context0); }
    bool in_evalf(int level,gen & evaled,const giac::context * contextptr) const;
    gen evalf(int level,const giac::context * contextptr) const;
    // inline gen evalf() const { return evalf(DEFAULT_EVAL_LEVEL,context0); }
    gen evalf_double(int level,const giac::context * contextptr) const ;
    gen evalf2double(int level,const giac::context * contextptr) const;
    %rename(operator_equal) gen::operator =;
    // gen & operator = (const gen & a);
    int to_int() const ;
    bool is_real(giac::context * contextptr) const ;
    bool is_cinteger() const ;
    bool is_integer() const ;
    bool is_constant() const;
    std::string print(giac::context * contextptr) const;
    const char * printcharptr() const ;
    // if sptr==0, return length required, otherwise print at end of *sptr
    int sprint(std::string * sptr,giac::context * contextptr) const; 
    std::string print_universal(giac::context * contextptr) const;
    std::string print() const;
    wchar_t * wprint(giac::context * contextptr) const ; 
    // print then convert to a malloc-ated wchar_t *
    void modify(int i) { *this =gen(i); };
    void dbgprint() const; 
    void uncoerce() ;
    gen conj(giac::context * contextptr) const;
    gen re(giac::context * contextptr) const ;
    gen im(giac::context * contextptr) const ;
    gen inverse(giac::context * contextptr) const;
    gen squarenorm(giac::context * contextptr) const;
    int bindigits() const ;
    // gen operator [] (int i) const ;
    // gen operator [] (const gen & i) const;
    gen operator_at(int i,giac::context * contextptr) const;
    gen operator_at(const gen & i,giac::context * contextptr) const;
    // gen & operator [] (int i) ;
    // gen & operator [] (const gen & i) ;
    %rename(operator_of) operator () (const gen &,giac::context *) const;
    gen operator () (const gen & i,giac::context * contextptr) const;
    // gen operator () (const gen & i,const gen & progname,giac::context * contextptr) const;
    bool islesscomplexthan(const gen & other) const;
    bool is_approx() const ; // true if double/real or cmplx with re/im
    int symb_size() const;
    gen change_subtype(int newsubtype);
    bool is_symb_of_sommet(const giac::unary_function_ptr & u) const ;
    // bool is_symb_of_sommet(const giac::unary_function_ptr * u) const ;
    gen makegen(int i) const; // make a gen of same type as this with integer i
  };

  gen _factor(const gen & g,giac::context * contextptr);
  gen _simplify(const gen & g,giac::context * contextptr);
  gen _size(const gen & g,giac::context * contextptr);
  %rename(add) operator + (const gen &,const gen &);
  gen operator + (const gen & a,const gen & b);
  %rename(sub) operator - (const gen &,const gen &);
  gen operator - (const gen & a,const gen & b);
  %rename(neg) operator - (const gen &);
  gen operator - (const gen & a);
  %rename(mul) operator * (const gen &,const gen &);
  gen operator * (const gen & a,const gen & b);
  gen rdiv(const gen & a,const gen & b); // rational division
  %rename(mod) operator % (const gen &,const gen &);
  gen operator %(const gen & a,const gen & b); // for int only
  gen pow(const gen & base,const gen & exponent,giac::context *);
  gen giac_pow(const gen & base,const gen & exponent,giac::context *);
  gen iquo(const gen & a,const gen & b); // same
  gen irem(const gen & a,const gen & b,gen & q); // same
  gen smod(const gen & a,const gen & b); // same
  void smod(const giac::vecteur & v,const gen & g,giac::vecteur & w); 
  giac::vecteur smod(const giac::vecteur & a,const gen & b); // same
  // gen inv(const gen & a);
  gen inv(const gen & a,giac::context *);
  bool operator_equal(const gen & a,const gen & b,giac::context *);
  gen equal(const gen & a,const gen &b,giac::context *);
  int fastsign(const gen & a,giac::context *);   // 0 if unknown, 1 if >0, -1 if <0
  gen sign(const gen & a,giac::context *);
  bool is_greater(const gen & a,const gen &b,giac::context *);
  bool is_strictly_greater(const gen & a,const gen &b,giac::context *);
  bool is_positive(const gen & a,giac::context *);
  bool is_strictly_positive(const gen & a,giac::context *);
  // Large tests if strictly not precised, if sign is unknown make an error
  bool ck_is_greater(const gen & a,const gen &b,giac::context *);
  bool ck_is_strictly_greater(const gen & a,const gen &b,giac::context *);
  bool ck_is_positive(const gen & a,giac::context *);
  bool ck_is_strictly_positive(const gen & a,giac::context *);
  gen superieur_strict(const gen & a,const gen & b,giac::context *);
  gen superieur_egal(const gen & a,const gen & b,giac::context *);
  gen inferieur_strict(const gen & a,const gen & b,giac::context *);
  gen inferieur_egal(const gen & a,const gen & b,giac::context *);
  bool symb_size_less(const gen & a,const gen & b);

  gen min(const gen & a, const gen & b,giac::context *);
  gen max(const gen & a, const gen & b,giac::context *);
  // default context0 is required for instantiation in poly.h
  gen factorial(unsigned long int i);
  gen comb(unsigned long int i,unsigned long j);
  gen perm(unsigned long int i,unsigned long j);
  gen pow(const gen & base, unsigned long int exponent);
  gen pow(const gen & base, int exponent);
  gen pow(unsigned long int base, unsigned long int exponent);

  // more advanced arithmetic
  gen gcd(const gen & A,const gen & B);
  gen lcm(const gen & a,const gen & b);
  gen simplify(gen & n, gen & d);
  void egcd(const gen &a,const gen &b, gen & u,gen &v,gen &d );
  gen ichinrem(const gen & a,const gen &b,const gen & amod, const gen & bmod);
  gen invmod(const gen & A,const gen & modulo);
  gen fracmod(const gen & a_orig,const gen & modulo); // -> p/q=a mod modulo
  bool fracmod(const gen & a_orig,const gen & modulo,gen & res);
  gen powmod(const gen &base,const gen & expo,const gen & modulo);
  gen isqrt(const gen & A);
  gen re(const gen & a,giac::context *);
  gen no_context_re(const gen & a);
  gen im(const gen & a,giac::context *);
  gen no_context_im(const gen & a);
  void reim(const gen & g,gen & r,gen & i,giac::context *);
  gen conj(const gen & a,giac::context *);
  gen no_context_conj(const gen & a);
  gen sq(const gen & a);
  gen abs(const gen & a,const context * contextptr=context0);
  // default context0 is required for instantiation in poly.h
  gen linfnorm(const gen & a,const context * contextptr=context0);
  // default context0 is required for instantiation in poly.h
  gen arg(const gen & a,giac::context *);
  gen arg_CPLX(const gen & a,giac::context *);
  int is_perfect_square(const gen & A);
  int is_probab_prime_p(const gen & A);
  gen nextprime(const gen & a); // more precisely next probably prime
  gen prevprime(const gen & a); // more precisely prev probably prime
  int jacobi(const gen & A, const gen &B);
  int legendre(const gen & A, const gen & B);
  giac::vecteur pascal_next_line(const giac::vecteur & v); 
  giac::vecteur pascal_nth_line(int n);
  // convert a __VECTOR__VECT vecteur to a normal vecteur
  gen vector2vecteur(const giac::vecteur & v);

  // if b is a _MOD, returns a as a b _MOD 
  gen chkmod(const gen& a,const gen & b);
  // make a _MOD a%b
  gen makemod(const gen & a,const gen & b);
  // same without evaluating %
  gen makemodquoted(const gen & a,const gen & b);

  // from a sum in x returns a list of [coeff monomial]
  // e.g. 5+2x+3*x*y -> [ [5 1] [2 x] [ 3 x*y] ]
  giac::vecteur symbolique2liste(const gen & x,giac::context *);
  // v should be sorted and shrinked
  gen liste2symbolique(const giac::vecteur & v);

  bool is_atomic(const gen & e);
  gen string2gen(const std::string & ss,bool remove_ss_quotes=true);
  // gen eval_VECT(const giac::vecteur & v,int subtype,int level,const context * context_ptr );
  gen no_context_evalf(const gen & e);
  gen evalf(const gen & e,int level,const context * contextptr );
  gen evalf2bcd_nock(const gen & g0,int level,const context * contextptr);
  gen evalf2bcd(const gen & g0,int level,const context * contextptr);
  // return true if g can be converted to a double or real or complex
  bool has_evalf(const gen & g,gen & res,int level,const context * contextptr);
  double int2double(int i);
  gen aplatir_fois_plus(const gen & g);
  gen collect(const gen & g,giac::context *);

}


